
package Rcam;
use warnings;
use strict;
use Object::Tiny qw{
 host_name
 cmd_path
 media_base_path
 smb
 local
};

use Filesys::SmbClient; # for talking to remote hosts
use File::Copy;
use Sys::Hostname; # for knowing if a host is remote or not (makes bareword hostname into the current host's name, weird)

#@@@ assumptions!
# the machine running this has set up /var/lib/FusorFiles with subdirs of the machine names with which
# this will be used, and they're writeable by "anybody"
# all vid cap machines are setup so the cam software is at /var/www/html with the otherwise normal structure
# and FIFO in that dir is world writeable.
# end_box.sh maks the cap.txt file under /var/www/html/media once the vid is ready for copy
# I will time how long it takes for the "local" machine to box a video, wait that long, and do it first,
# giving time (I hope) for the other slower machines to complete before we try them.
# Should save a bunch of stat()ing and waiting around and checking. 
# EG beware the code that isn't there as it won't need to be.

# cap.txt in media_base_path names the last video when boxing is done (some time after stop, we'll have to wait a bit)
# host_name is used as a folder name under /var/lib/FusorFiles to copy captured video to
# cmd_path is to wherever the pi's FIFO is for starting and stopping vid cap via writing ca 1 to start, ca 0 to stop

# we strip off vi_number before the rest as it's a meaningless sequence number for us,
# and that filename is already ugly-long (should sort better without it too).

my $debug = 0; # class wide

##############################################################################
sub init() { # make sure samba and network are ok, build paths for later on
my $self = shift;
die "Can't connect to a null hostname!" unless $self->{'host_name'}; # if not given at create time
# check if we want to do this to "ourself", the rest of the code is different (no need or desire for SMB in that case)
$self->{local} = '0'; # in case some bozo sets it in constructor
my $host = hostname; # ask module where we are running - weird bareword use from it
# debug print "host: $host passed: $self->{'host_name'}\n";
if ($host =~ m/$self->{host_name}/i) # we are local
{
# print "matched\n";
 $self->{local} = 1; # tell ourself the status
 $self->{cmd_path} = '/var/www/html/FIFO'; # assumes our standard setup for file locations in pi cam
 $self->{media_base_path} = '/var/www/html/media/';
} else { # we need samba
 $self->{smb} = new Filesys::SmbClient(username => "pi",password => "dotdot", workgroup => "MOUNTAIN"); # be pi
#note this works fine with a bullshit host name passed in, as we don't actually connect to it yet
 die "Couldn't connect to network." unless $self->{smb};
 $self->{cmd_path} = 'smb://' . $self->{host_name} . '/cam/FIFO'; # go ahead and build these strings
 $self->{media_base_path} = 'smb://' . $self->{host_name} . '/media/'; # when it isn't time-critical
 }
# implied true return if any of it worked
}
##############################################################################
sub start_cap() { # begin recording video at target machine
my $self = shift;
my $fd;
if ($self->{local}) {
# has to be tested on a pi that has a cam
   open($fd,">", "$self->{cmd_path}")  or die "couldn't open FIFO $!";
   print $fd "ca 1";
   close $fd;
 } else { # use smb
  $fd = $self->{smb}->open(">" . "$self->{cmd_path}")  or die "couldn't open FIFO $!";
  $self->{smb}->write($fd, "ca 1") or die "couldn't write FIFO $!";  #ca 1 is the command to capture
  $self->{smb}->close($fd);
 }
}
##############################################################################
sub end_cap() { # stop recording on target machine, begin boxing
my $self = shift;
my $fd;
if ($self->{local}) {
   open($fd,">", "$self->{cmd_path}")  or die "couldn't open FIFO $!";
   print $fd "ca 0";
   close $fd;
 } else { # use smb
  $fd = $self->{smb}->open(">" . "$self->{cmd_path}")  or die "couldn't open FIFO $!";
  $self->{smb}->write($fd, "ca 0") or die "couldn't write FIFO $!";  #ca 0 is the command to stop
  $self->{smb}->close($fd);

 }
}
##############################################################################
sub copy_vid() { # get video file post-boxing...@@@timing?
# destination path /var/lib/FusorFiles/host_name/filename
my $self = shift;
my $sfd;
my $readdat;
my $sourcepath = $self->{media_base_path}; 
my $destination = '/var/lib/FusorFiles/' . "$self->{host_name}" . '/'; # all but vid filename
if ($self->{local}) {
  # do local copy here
   open($sfd, "<", "$self->{media_base_path}" . 'cap.txt');
   $readdat = <$sfd>;
   close $sfd;
   chomp $readdat;
   $sourcepath .= $readdat; # path to source video
   print "sourcepath: $sourcepath\n" if ($debug);
   $readdat =~ s/vi_\d+_//; # remove leading v_sequence#_
   $destination .= $readdat;
   print "destination: $destination\n" if ($debug);
   copy($sourcepath,$destination) or die "vid copy failed: $!";
   return $destination;
 } else { # use smb to do it
  local *FD;
  tie(*FD,'Filesys::SmbClient');
  open(FD,'<' . "$self->{media_base_path}" . 'cap.txt');
  $readdat = <FD>;
  close (FD); # be clean
  chomp $readdat;
  $sourcepath .= $readdat; # path to source video
  print "sourcepath: $sourcepath\n" if ($debug);
  $readdat =~ s/vi_\d+_//; # remove leading v_sequence#_
  $destination .= $readdat;
  print "destination: $destination\n" if ($debug);

   tie(*FD,'Filesys::SmbClient');
   open(FD,'<' . "$sourcepath") or die "couldn't open $sourcepath:$!";
  #maybe write my own copy and use fstat from SmbClient?
  # warns but works.
   no warnings;
   copy(*FD,"$destination") or die "copy failed: $!";
   close(FD);
   return $destination;
 } # smb copy
}

1; # the usual return true from modules/classes
